@@ -73,7 +73,9 @@ static void meta_frames_ensure_layout (MetaFrames      *frames,
 static MetaUIFrame* meta_frames_lookup_window (MetaFrames *frames,
                                                Window      xwindow);
 
-static void meta_frames_font_changed (MetaFrames *frames);
+static void meta_frames_font_changed          (MetaFrames *frames);
+static void meta_frames_button_layout_changed (MetaFrames *frames);
+
 
 static GdkRectangle*    control_rect (MetaFrameControl   control,
                                       MetaFrameGeometry *fgeom);
@@ -161,12 +163,19 @@ unsigned_long_hash (gconstpointer v)
 }
 
 static void
-font_changed_callback (MetaPreference pref,
-		       void          *data)
+prefs_changed_callback (MetaPreference pref,
+                        void          *data)
 {
-  if (pref == META_PREF_TITLEBAR_FONT)
+  switch (pref)
     {
+    case META_PREF_TITLEBAR_FONT:
       meta_frames_font_changed (META_FRAMES (data));
+      break;
+    case META_PREF_BUTTON_LAYOUT:
+      meta_frames_button_layout_changed (META_FRAMES (data));
+      break;
+    default:
+      break;
     }
 }
 
@@ -185,7 +194,7 @@ meta_frames_init (MetaFrames *frames)
 
   gtk_widget_set_double_buffered (GTK_WIDGET (frames), FALSE);
 
-  meta_prefs_add_listener (font_changed_callback, frames);
+  meta_prefs_add_listener (prefs_changed_callback, frames);
 }
 
 static void
@@ -237,7 +246,7 @@ meta_frames_finalize (GObject *object)
   
   frames = META_FRAMES (object);
 
-  meta_prefs_remove_listener (font_changed_callback, frames);
+  meta_prefs_remove_listener (prefs_changed_callback, frames);
   
   g_hash_table_destroy (frames->text_heights);
   
@@ -292,6 +301,31 @@ meta_frames_font_changed (MetaFrames *frames)
 
 }
 
+static void
+queue_draw_func (gpointer key, gpointer value, gpointer data)
+{
+  MetaUIFrame *frame;
+  MetaFrames *frames;
+
+  frames = META_FRAMES (data);
+  frame = value;
+
+  /* If a resize occurs it will cause a redraw, but the
+   * resize may not actually be needed so we always redraw
+   * in case of color change.
+   */
+  gtk_style_set_background (GTK_WIDGET (frames)->style,
+                            frame->window, GTK_STATE_NORMAL);
+  gdk_window_invalidate_rect (frame->window, NULL, FALSE);
+}
+
+static void
+meta_frames_button_layout_changed (MetaFrames *frames)
+{
+  g_hash_table_foreach (frames->frames,
+                        queue_draw_func, frames);
+}
+
 static void
 meta_frames_style_set  (GtkWidget *widget,
                         GtkStyle  *prev_style)
@@ -394,6 +428,7 @@ meta_frames_calc_geometry (MetaFrames        *frames,
   int width, height;
   MetaFrameFlags flags;
   MetaFrameType type;
+  MetaButtonLayout button_layout;
   
   meta_core_get_client_size (gdk_display, frame->xwindow,
                              &width, &height);
@@ -402,12 +437,15 @@ meta_frames_calc_geometry (MetaFrames        *frames,
   type = meta_core_get_frame_type (gdk_display, frame->xwindow);
 
   meta_frames_ensure_layout (frames, frame);
+
+  meta_prefs_get_button_layout (&button_layout);
   
   meta_theme_calc_geometry (meta_theme_get_current (),
                             type,
                             frame->text_height,
                             flags,
                             width, height,
+                            &button_layout,
                             fgeom);
 }
 
@@ -1030,18 +1068,41 @@ meta_frames_button_press_event (GtkWidget      *widget,
     {
       MetaFrameFlags flags;
       
-      flags = meta_core_get_frame_flags (gdk_display, frame->xwindow);
-
-      if (flags & META_FRAME_ALLOWS_SHADE)
+      switch (meta_prefs_get_action_double_click_titlebar ())
         {
-          if (flags & META_FRAME_SHADED)
-            meta_core_unshade (gdk_display,
-                               frame->xwindow);
-          else
-            meta_core_shade (gdk_display,
-                             frame->xwindow);
-        }
+        case META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_SHADE:
+          {
+            flags = meta_core_get_frame_flags (gdk_display, frame->xwindow);
+            
+            if (flags & META_FRAME_ALLOWS_SHADE)
+              {
+                if (flags & META_FRAME_SHADED)
+                  meta_core_unshade (gdk_display,
+                                     frame->xwindow);
+                else
+                  meta_core_shade (gdk_display,
+                                   frame->xwindow);
+              }
+          }
+          break;          
+          
+        case META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_MAXIMIZE:
+          {
+            MetaFrameFlags flags;
+            
+            flags = meta_core_get_frame_flags (gdk_display, frame->xwindow);
+            
+            if (flags & META_FRAME_ALLOWS_MAXIMIZE)
+              {
+                meta_core_toggle_maximize (gdk_display, frame->xwindow);
+              }
+          }
+          break;
 
+        case META_ACTION_DOUBLE_CLICK_TITLEBAR_LAST:
+          break;
+        }
+      
       return TRUE;
     }
 
@@ -1169,8 +1230,7 @@ meta_frames_button_press_event (GtkWidget      *widget,
                                event->x_root,
                                event->y_root);
     }
-  else if ((control == META_FRAME_CONTROL_TITLE ||
-            control == META_FRAME_CONTROL_NONE) &&
+  else if (control == META_FRAME_CONTROL_TITLE &&
            event->button == 1)
     {
       MetaFrameFlags flags;
@@ -1190,6 +1250,10 @@ meta_frames_button_press_event (GtkWidget      *widget,
                                    event->y_root);
         }
     }
+  else if (event->button == 2)
+    {
+      meta_core_user_lower (gdk_display, frame->xwindow);
+    }
   else if (event->button == 3)
     {
       meta_core_show_window_menu (gdk_display,
@@ -1524,6 +1588,7 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
   GdkRectangle *areas;
   int n_areas;
   int screen_width, screen_height;
+  MetaButtonLayout button_layout;
   
   widget = GTK_WIDGET (frames);
 
@@ -1656,10 +1721,12 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
 
   /* Now draw remaining portion of region */
   gdk_region_get_rectangles (edges, &areas, &n_areas);
+
+  meta_prefs_get_button_layout (&button_layout);
   
   i = 0;
   while (i < n_areas)
-    {      
+    {
       if (GDK_IS_WINDOW (drawable))
         gdk_window_begin_paint_rect (drawable, &areas[i]);
       
@@ -1673,6 +1740,7 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
                              w, h,
                              frame->layout,
                              frame->text_height,
+                             &button_layout,
                              button_states,
                              mini_icon, icon);
 
@@ -1787,14 +1855,17 @@ get_control (MetaFrames *frames,
   MetaFrameFlags flags;
   gboolean has_vert, has_horiz;
   GdkRectangle client;
+  int bottom_of_titlebar;
   
   meta_frames_calc_geometry (frames, frame, &fgeom);
 
   client.x = fgeom.left_width;
   client.y = fgeom.top_height;
   client.width = fgeom.width - fgeom.left_width - fgeom.right_width;
-  client.height = fgeom.height - fgeom.top_height - fgeom.bottom_height;
+  client.height = fgeom.height - fgeom.top_height - fgeom.bottom_height;  
 
+  bottom_of_titlebar = fgeom.title_rect.y + fgeom.title_rect.height;
+  
   if (POINT_IN_RECT (x, y, client))
     return META_FRAME_CONTROL_CLIENT_AREA;
   
@@ -1806,16 +1877,19 @@ get_control (MetaFrames *frames,
 
   if (POINT_IN_RECT (x, y, fgeom.menu_rect))
     return META_FRAME_CONTROL_MENU;
+
+  flags = meta_core_get_frame_flags (gdk_display, frame->xwindow);
+  
+  has_vert = (flags & META_FRAME_ALLOWS_VERTICAL_RESIZE) != 0;
+  has_horiz = (flags & META_FRAME_ALLOWS_HORIZONTAL_RESIZE) != 0;
   
   if (POINT_IN_RECT (x, y, fgeom.title_rect))
     {
-      if (y <= TOP_RESIZE_HEIGHT)
+      if (has_vert && y <= TOP_RESIZE_HEIGHT)
         return META_FRAME_CONTROL_RESIZE_N;
       else
         return META_FRAME_CONTROL_TITLE;
     }
-  
-  flags = meta_core_get_frame_flags (gdk_display, frame->xwindow);
 
   if (POINT_IN_RECT (x, y, fgeom.max_rect))
     {
@@ -1824,83 +1898,78 @@ get_control (MetaFrames *frames,
       else
         return META_FRAME_CONTROL_MAXIMIZE;
     }
-  
-  has_vert = (flags & META_FRAME_ALLOWS_VERTICAL_RESIZE) != 0;
-  has_horiz = (flags & META_FRAME_ALLOWS_HORIZONTAL_RESIZE) != 0;
-
-  if (has_vert || has_horiz)
-    {
-      int bottom_of_titlebar;
-
-      bottom_of_titlebar = fgeom.title_rect.y + fgeom.title_rect.height;
       
-      /* South resize always has priority over north resize,
-       * in case of overlap.
-       */
+  /* South resize always has priority over north resize,
+   * in case of overlap.
+   */
 
-      if (y >= (fgeom.height - fgeom.bottom_height - RESIZE_EXTENDS) &&
-          x >= (fgeom.width - fgeom.right_width - RESIZE_EXTENDS))
-        {
-          if (has_vert && has_horiz)
-            return META_FRAME_CONTROL_RESIZE_SE;
-          else if (has_vert)
-            return META_FRAME_CONTROL_RESIZE_S;
-          else
-            return META_FRAME_CONTROL_RESIZE_E;
-        }
-      else if (y >= (fgeom.height - fgeom.bottom_height - RESIZE_EXTENDS) &&
-               x <= (fgeom.left_width + RESIZE_EXTENDS))
-        {
-          if (has_vert && has_horiz)
-            return META_FRAME_CONTROL_RESIZE_SW;
-          else if (has_vert)
-            return META_FRAME_CONTROL_RESIZE_S;
-          else
-            return META_FRAME_CONTROL_RESIZE_W;
-        }
-      else if (y < (fgeom.top_height + RESIZE_EXTENDS) &&
-               x < RESIZE_EXTENDS)
-        {
-          if (has_vert && has_horiz)
-            return META_FRAME_CONTROL_RESIZE_NW;
-          else if (has_vert)
-            return META_FRAME_CONTROL_RESIZE_N;
-          else
-            return META_FRAME_CONTROL_RESIZE_W;
-        }
-      else if (y < (fgeom.top_height + RESIZE_EXTENDS) &&
-               x >= (fgeom.width - RESIZE_EXTENDS))
-        {
-          if (has_vert && has_horiz)
-            return META_FRAME_CONTROL_RESIZE_NE;
-          else if (has_vert)
-            return META_FRAME_CONTROL_RESIZE_N;
-          else
-            return META_FRAME_CONTROL_RESIZE_E;
-        }
-      else if (y >= (fgeom.height - fgeom.bottom_height - RESIZE_EXTENDS))
-        {
-          if (has_vert)
-            return META_FRAME_CONTROL_RESIZE_S;
-        }
-      else if (y <= TOP_RESIZE_HEIGHT)
-        {
-          if (has_vert)
-            return META_FRAME_CONTROL_RESIZE_N;
-        }
-      else if (x <= fgeom.left_width)
-        {
-          if (has_horiz)
-            return META_FRAME_CONTROL_RESIZE_W;
-        }
-      else if (x >= (fgeom.width - fgeom.right_width))
-        {
-          if (has_horiz)
-            return META_FRAME_CONTROL_RESIZE_E;
-        }
+  if (y >= (fgeom.height - fgeom.bottom_height - RESIZE_EXTENDS) &&
+      x >= (fgeom.width - fgeom.right_width - RESIZE_EXTENDS))
+    {
+      if (has_vert && has_horiz)
+        return META_FRAME_CONTROL_RESIZE_SE;
+      else if (has_vert)
+        return META_FRAME_CONTROL_RESIZE_S;
+      else if (has_horiz)
+        return META_FRAME_CONTROL_RESIZE_E;
     }
-  
-  return META_FRAME_CONTROL_NONE;
+  else if (y >= (fgeom.height - fgeom.bottom_height - RESIZE_EXTENDS) &&
+           x <= (fgeom.left_width + RESIZE_EXTENDS))
+    {
+      if (has_vert && has_horiz)
+        return META_FRAME_CONTROL_RESIZE_SW;
+      else if (has_vert)
+        return META_FRAME_CONTROL_RESIZE_S;
+      else if (has_horiz)
+        return META_FRAME_CONTROL_RESIZE_W;
+    }
+  else if (y < (fgeom.top_height + RESIZE_EXTENDS) &&
+           x < RESIZE_EXTENDS)
+    {
+      if (has_vert && has_horiz)
+        return META_FRAME_CONTROL_RESIZE_NW;
+      else if (has_vert)
+        return META_FRAME_CONTROL_RESIZE_N;
+      else if (has_horiz)
+        return META_FRAME_CONTROL_RESIZE_W;
+    }
+  else if (y < (fgeom.top_height + RESIZE_EXTENDS) &&
+           x >= (fgeom.width - RESIZE_EXTENDS))
+    {
+      if (has_vert && has_horiz)
+        return META_FRAME_CONTROL_RESIZE_NE;
+      else if (has_vert)
+        return META_FRAME_CONTROL_RESIZE_N;
+      else if (has_horiz)
+        return META_FRAME_CONTROL_RESIZE_E;
+    }
+  else if (y >= (fgeom.height - fgeom.bottom_height - RESIZE_EXTENDS))
+    {
+      if (has_vert)
+        return META_FRAME_CONTROL_RESIZE_S;
+    }
+  else if (y <= TOP_RESIZE_HEIGHT)
+    {
+      if (has_vert)
+        return META_FRAME_CONTROL_RESIZE_N;
+      else if (has_horiz)
+        return META_FRAME_CONTROL_TITLE;
+    }
+  else if (x <= fgeom.left_width)
+    {
+      if (has_horiz)
+        return META_FRAME_CONTROL_RESIZE_W;
+    }
+  else if (x >= (fgeom.width - fgeom.right_width))
+    {
+      if (has_horiz)
+        return META_FRAME_CONTROL_RESIZE_E;
+    }
+
+  if (y >= bottom_of_titlebar)
+    return META_FRAME_CONTROL_NONE;
+  else
+    return META_FRAME_CONTROL_TITLE;
 }
 
 void
