@@ -215,26 +215,71 @@ main (int argc, char **argv)
   return 0;
 }
 
+static void
+response_cb (GtkDialog *dialog,
+             int        response_id,
+             void      *data);
+
+static void
+make_dialog (GtkWidget *parent,
+             int        depth)
+{
+  GtkWidget *dialog;
+  char *str;
+  
+  dialog = gtk_message_dialog_new (parent ? GTK_WINDOW (parent) : NULL,
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_INFO,
+                                   GTK_BUTTONS_CLOSE,
+                                   parent ? "Here is a dialog %d" :
+                                   "Here is a dialog %d with no transient parent",
+                                   depth);
+
+  str = g_strdup_printf ("%d dialog", depth);
+  gtk_window_set_title (GTK_WINDOW (dialog), str);
+  g_free (str);
+  
+  gtk_dialog_add_button (GTK_DIALOG (dialog),
+                         "Open child dialog",
+                         GTK_RESPONSE_ACCEPT);
+  
+  /* Close dialog on user response */
+  g_signal_connect (G_OBJECT (dialog),
+                    "response",
+                    G_CALLBACK (response_cb),
+                    NULL);
+
+  g_object_set_data (G_OBJECT (dialog), "depth",
+                     GINT_TO_POINTER (depth));
+  
+  gtk_widget_show (dialog);
+}
+
+static void
+response_cb (GtkDialog *dialog,
+             int        response_id,
+             void      *data)
+{
+  switch (response_id)
+    {
+    case GTK_RESPONSE_ACCEPT:
+      make_dialog (GTK_WIDGET (dialog),
+                   GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog),
+                                                       "depth")) + 1);
+      break;
+
+    default:
+      gtk_widget_destroy (GTK_WIDGET (dialog));
+      break;
+    }
+}
+
 static void
 dialog_cb (gpointer             callback_data,
            guint                callback_action,
            GtkWidget           *widget)
 {
-  GtkWidget *dialog;
-  
-  dialog = gtk_message_dialog_new (GTK_WINDOW (callback_data),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_CLOSE,
-                                   "Here is a dialog");
-
-  /* Close dialog on user response */
-  g_signal_connect (G_OBJECT (dialog),
-                    "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-  
-  gtk_widget_show (dialog);
+  make_dialog (GTK_WIDGET (callback_data), 1);
 }
 
 static void
@@ -262,21 +307,7 @@ no_parent_dialog_cb (gpointer             callback_data,
                      guint                callback_action,
                      GtkWidget           *widget)
 {
-  GtkWidget *dialog;
-  
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_CLOSE,
-                                   "Here is a dialog with no transient parent");
-
-  /* Close dialog on user response */
-  g_signal_connect (G_OBJECT (dialog),
-                    "response",
-                    G_CALLBACK (gtk_widget_destroy),
-                    NULL);
-  
-  gtk_widget_show (dialog);
+  make_dialog (NULL, 1);
 }
 
 static void
